import unittest
import os
from packagepoa import decapitate_pdf


class TestDecapitate(unittest.TestCase):
    def setUp(self):
        pass

    def test_decapitate_pdf_no_executable(self):
        "set of tests building csv into xml and compare the output"
        # override the config first
        poa_config = None
        return_value = decapitate_pdf.decapitate_pdf_with_error_check(
            None, None, poa_config
        )
        self.assertFalse(return_value)

    def test_decapitate_pdf(self):
        "mock the process to test logging"
        poa_config = {"strip_coverletter_executable": "tests/test.sh"}
        pdf_in = os.path.join("test_data", "decap_elife_poa_212717.pdf")
        pdf_out_dir = "tmp"
        return_value = decapitate_pdf.decapitate_pdf_with_error_check(
            pdf_in, pdf_out_dir, poa_config
        )
        self.assertEqual(return_value, True)



from __future__ import print_function
import logging
import subprocess

FORMAT = logging.Formatter(
    "%(created)f - %(levelname)s - %(processName)s - %(name)s - %(message)s"
)
LOGFILE = "decapitate_pdf.log"
LOGGER = logging.getLogger(__file__)
LOGGER.setLevel(logging.DEBUG)
HDLR = logging.FileHandler(LOGFILE)
HDLR.setLevel(logging.INFO)
HDLR.setFormatter(FORMAT)
LOGGER.addHandler(HDLR)

PDF_EXECUTABLE_DEFAULT = "/opt/strip-coverletter/strip-coverletter-docker.sh"


def decapitate_pdf_with_error_check(pdf_in, pdf_out_dir, poa_config=None):
    # configuration
    pdf_executable = None
    if poa_config:
        pdf_executable = poa_config.get("strip_coverletter_executable")
    if not pdf_executable:
        return False

    # PDF out file name
    pdf_out = pdf_out_dir + pdf_in.split("/")[-1]

    process = subprocess.Popen(
        [pdf_executable, pdf_in, pdf_out],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )

    stderr = process.stderr.read()
    stdout = process.stdout.read()

    # subprocess.Popen doesn't interleave pipe output,
    # so neither will these log messages
    if stdout:
        [LOGGER.info(line.decode("utf-8")) for line in stdout.splitlines()]
    if stderr:
        [LOGGER.error(line.decode("utf-8")) for line in stderr.splitlines()]

    return stderr != ""  # no errors


if __name__ == "__main__":
    import sys

    ARGS = sys.argv[1:]
    if len(ARGS) < 2:
        print("Usage: decapitate_pdf.py <pdf-in> <pdf-out>")
        exit(1)

    HANDLER = logging.StreamHandler()
    HANDLER.setLevel(logging.DEBUG)
    HANDLER.setFormatter(FORMAT)
    LOGGER.addHandler(HANDLER)

    PIN = ARGS[0]
    POUT = ARGS[1]
    CONFIG = {"strip_coverletter_executable": PDF_EXECUTABLE_DEFAULT}
    decapitate_pdf_with_error_check(PIN, POUT, CONFIG)



import unittest
import shutil
import os
import zipfile
from mock import patch
from func_timeout import FunctionTimedOut
from packagepoa import transform
from packagepoa.conf import raw_config, parse_raw_config

POA_CONFIG = parse_raw_config(raw_config("elife"))
TEST_BASE_PATH = os.path.dirname(os.path.abspath(__file__)) + os.sep
TEST_DATA_PATH = TEST_BASE_PATH + "test_data" + os.sep
POA_CONFIG["tmp_dir"] = TEST_BASE_PATH + POA_CONFIG["tmp_dir"]
POA_CONFIG["output_dir"] = TEST_BASE_PATH + POA_CONFIG["output_dir"]
POA_CONFIG["decapitate_pdf_dir"] = TEST_BASE_PATH + POA_CONFIG["decapitate_pdf_dir"]


def mock_decapitate_pdf(filename):
    "copy a file to simulate the PDF decapitation process"
    from_filename = os.path.join(TEST_DATA_PATH, filename)
    to_filename = os.path.join(POA_CONFIG.get("decapitate_pdf_dir"), filename)
    shutil.copy(from_filename, to_filename)


def list_test_dir(dir_name, ignore=(".keepme")):
    "list the contents of a directory ignoring the ignore files"
    file_names = os.listdir(dir_name)
    return [file_name for file_name in file_names if file_name not in ignore]


def clean_test_dir(dir_name, ignore=(".keepme")):
    "clean files from a test directory ignoring the .keepme file"
    file_names = list_test_dir(dir_name, ignore)
    for file_name in file_names:
        os.remove(os.path.join(dir_name, file_name))


def clean_test_directories(ignore=(".keepme")):
    "clean each of the testing directories"
    dir_names = [
        POA_CONFIG.get("tmp_dir"),
        POA_CONFIG.get("output_dir"),
        POA_CONFIG.get("decapitate_pdf_dir"),
    ]
    for dir_name in dir_names:
        clean_test_dir(dir_name, ignore)


class TestTransform(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        # clean up the test directories
        clean_test_directories()

    @patch.object(transform, "decapitate_pdf_with_error_check")
    def test_process_zipfile(self, fake_decapitate):
        mock_decapitate_pdf("decap_elife_poa_e12717.pdf")
        fake_decapitate.return_value = True
        zipfile_name = os.path.join(
            TEST_DATA_PATH, "18022_1_supp_mat_highwire_zip_268991_x75s4v.zip"
        )
        # pass in the POA_CONFIG which has the testing directories modified
        return_value = transform.process_zipfile(zipfile_name, POA_CONFIG)
        # check return value
        self.assertTrue(return_value)
        # check directory contents
        self.assertEqual(
            sorted(list_test_dir(POA_CONFIG.get("tmp_dir"))),
            ["decap_elife_poa_e12717.pdf", "temp_transfer"],
        )
        self.assertEqual(
            sorted(list_test_dir(POA_CONFIG.get("decapitate_pdf_dir"))),
            ["decap_elife_poa_e12717.pdf"],
        )
        self.assertEqual(
            sorted(list_test_dir(POA_CONFIG.get("output_dir"))),
            ["elife_poa_e12717.pdf", "elife_poa_e12717_ds.zip"],
        )
        # check the ds zip contents
        zip_file_name = os.path.join(
            POA_CONFIG.get("output_dir"), "elife_poa_e12717_ds.zip"
        )
        with zipfile.ZipFile(zip_file_name, "r") as zip_file:
            self.assertEqual(
                sorted(zip_file.namelist()),
                [
                    "elife_poa_e12717_Figure_1_figure_supplement_1.pdf",
                    "elife_poa_e12717_Figure_2_figure_supplement_1.pdf",
                    "elife_poa_e12717_Figure_2_figure_supplement_2.pdf",
                    "elife_poa_e12717_Figure_3_figure_supplement_1.pdf",
                    "elife_poa_e12717_Figure_3_figure_supplement_2.pdf",
                    "elife_poa_e12717_Figure_3_figure_supplement_3.pdf",
                    "elife_poa_e12717_Figure_4_figure_supplement_1.pdf",
                    "elife_poa_e12717_Figure_4_figure_supplement_2.pdf",
                    "elife_poa_e12717_Figure_6_figure_supplement_1.pdf",
                    "elife_poa_e12717_Figure_6_figure_supplement_2.pdf",
                    "elife_poa_e12717_Figure_7_figure_supplement_1.pdf",
                    "elife_poa_e12717_Figure_8_figure_supplement_1.pdf",
                    "elife_poa_e12717_Supplementary_File_1.xls",
                    "elife_poa_e12717_Supplementary_File_2.xlsx",
                    "elife_poa_e12717_Supplementary_File_3.xls",
                    "elife_poa_e12717_Video_1.mov",
                    "elife_poa_e12717_Video_10.mov",
                    "elife_poa_e12717_Video_11.mov",
                    "elife_poa_e12717_Video_12.mov",
                    "elife_poa_e12717_Video_2.mov",
                    "elife_poa_e12717_Video_3.mov",
                    "elife_poa_e12717_Video_4.mov",
                    "elife_poa_e12717_Video_5.mov",
                    "elife_poa_e12717_Video_6.mov",
                    "elife_poa_e12717_Video_7.mov",
                    "elife_poa_e12717_Video_8.mov",
                    "elife_poa_e12717_Video_9.mov",
                ],
            )

    @patch.object(transform, "decapitate_pdf_with_error_check")
    def test_process_zipfile_failed_pdf(self, fake_decapitate):
        "tests of when pdf decapitaion fails, for test coverage"
        # first example is pdf decapitator returns False due to an error
        fake_decapitate.return_value = False
        zipfile_name = os.path.join(
            TEST_DATA_PATH, "18022_1_supp_mat_highwire_zip_268991_x75s4v.zip"
        )
        return_value = transform.process_zipfile(zipfile_name, None)
        # for now it still returns True
        self.assertTrue(return_value)
        # second example returns True but the pdf file is not found in the expected folder
        fake_decapitate.return_value = True
        zipfile_name = os.path.join(
            TEST_DATA_PATH, "18022_1_supp_mat_highwire_zip_268991_x75s4v.zip"
        )
        return_value = transform.process_zipfile(zipfile_name, None)
        # for now it still returns True
        self.assertTrue(return_value)

    @patch.object(transform, "decapitate_pdf_with_error_check")
    def test_copy_pdf_to_output_dir_timed_out_pdf(self, fake_decapitate):
        "tests of when pdf decapitaion reaches timeout"
        zipfile_name = os.path.join(
            TEST_DATA_PATH, "18022_1_supp_mat_highwire_zip_268991_x75s4v.zip"
        )
        file_title_map = {"18022_1_merged_1463214271.pdf": "Merged PDF"}
        fake_decapitate.side_effect = FunctionTimedOut
        doi = "10.7554/eLife.12717"
        with zipfile.ZipFile(zipfile_name, "r") as current_zipfile:
            with self.assertRaises(Exception):
                transform.copy_pdf_to_output_dir(
                    file_title_map, None, doi, current_zipfile, POA_CONFIG
                )

    def test_add_file_to_zipfile(self):
        "test adding files to a zip file"
        zip_file_name = os.path.join(POA_CONFIG.get("tmp_dir"), "test.zip")
        file_name = os.path.join(POA_CONFIG.get("tmp_dir"), ".keepme")
        # try to add a file not specifying a new name
        with zipfile.ZipFile(zip_file_name, "w") as zip_file:
            transform.add_file_to_zipfile(zip_file, file_name, None)
            self.assertEqual(zip_file.namelist(), [])



import configparser
import json

CONFIG_FILE = "packagepoa.cfg"
BOOLEAN_VALUES = []
INT_VALUES = []
LIST_VALUES = []


def load_config(config_file=CONFIG_FILE):
    config = configparser.ConfigParser(interpolation=None)
    config.read(config_file)
    return config


def raw_config(config_section, config_file=CONFIG_FILE):
    "try to load the config section"
    config = load_config(config_file)
    if config.has_section(config_section):
        return config[config_section]
    # default
    return config.defaults()


def boolean_config(config, value_name):
    "extract the value as a boolean"
    return config.getboolean(value_name)


def int_config(config, value_name):
    "extract the value as an int"
    return config.getint(value_name)


def list_config(config, value_name):
    "extract the value as a list parsed from JSON"
    return json.loads(config.get(value_name))


def parse_raw_config(config):
    "parse the raw config to something good"
    config_dict = {}

    for value_name in config:
        if value_name in BOOLEAN_VALUES:
            config_dict[value_name] = boolean_config(config, value_name)
        elif value_name in INT_VALUES:
            config_dict[value_name] = int_config(config, value_name)
        elif value_name in LIST_VALUES:
            config_dict[value_name] = list_config(config, value_name)
        else:
            # default
            config_dict[value_name] = config.get(value_name)
    return config_dict



from setuptools import setup
import packagepoa


with open("README.md") as fp:
    README = fp.read()


setup(
    name="packagepoa",
    version=packagepoa.__version__,
    description="Generate, transform and assemble files for a PoA article.",
    long_description=README,
    long_description_content_type="text/markdown",
    packages=["packagepoa"],
    license="MIT",
    install_requires=["configparser"],
    url="https://github.com/elifesciences/package-poa",
    maintainer="eLife Sciences Publications Ltd.",
    maintainer_email="tech-team@elifesciences.org",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
    ],
)



"""
open the zip file from EJP,
get the manifest.xml file
move the pdf
rename and move supp files to a new zipfile
find the pdf file and decapitate the cover page from it
move the PDF to the output directory
move the new zip file to the output directory
"""

import zipfile
import logging
import shutil
import os
from xml.etree import ElementTree
from func_timeout import func_timeout, FunctionTimedOut
from packagepoa.decapitate_pdf import decapitate_pdf_with_error_check
from packagepoa.conf import raw_config, parse_raw_config

# local logger
LOGGER = logging.getLogger("transform")
HDLR = logging.FileHandler("transform.log")
FORMATTER = logging.Formatter("%(asctime)s %(levelname)s %(message)s")
HDLR.setFormatter(FORMATTER)
LOGGER.addHandler(HDLR)
LOGGER.setLevel(logging.INFO)

# global logger
MANIFEST_LOGGER = logging.getLogger("manifest")
MANIFEST_HDLR = logging.FileHandler("manifest.log")
MANIFEST_HDLR.setFormatter(FORMATTER)
MANIFEST_LOGGER.addHandler(MANIFEST_HDLR)
MANIFEST_LOGGER.setLevel(logging.INFO)
PDF_DECAPITATE_TIMEOUT = 120


def article_id_from_doi(doi):
    article_id = doi.split(".")[-1]
    return article_id


def gen_new_name_for_file(name, title, doi, filename_pattern):
    """
    take the following:
    and generates a file name like:
    """
    file_ext = name.split(".")[1]
    article_id = article_id_from_doi(doi)
    new_name_front = title.replace(" ", "_")
    new_name_front = new_name_front.replace("-", "_")
    new_name_front = new_name_front.replace("__", "_")
    new_name_front = new_name_front.replace("__", "_")
    if new_name_front == "Merged_PDF":
        # we ignore the main file name and just use our base POA convention
        new_name = filename_pattern.format(
            article_id=article_id, extra="", file_ext=file_ext
        )
    else:
        new_name = filename_pattern.format(
            article_id=article_id, extra="_" + new_name_front, file_ext=file_ext
        )
    return new_name


def get_doi_from_zipfile(ejp_input_zipfile):
    # print ejp_input_zipfile.namelist()
    manifest = ejp_input_zipfile.read("manifest.xml")
    tree = ElementTree.fromstring(manifest)
    doi = None
    for child in tree:
        if child.tag == "resource":
            if child.attrib["type"] == "doi":
                doi = child.text
    return doi


def get_filename_new_title_map(ejp_input_zipfile):
    MANIFEST_LOGGER.info("unpacking and renaming %s", ejp_input_zipfile.filename)
    file_title_map = {}
    manifest = ejp_input_zipfile.read("manifest.xml")
    tree = ElementTree.fromstring(manifest)
    for child in tree:
        if child.tag == "file":
            for file_tag in child:
                if file_tag.tag == "filename":
                    filename = file_tag.text
                if file_tag.tag == "title":
                    title = file_tag.text
            file_title_map[filename] = title
    MANIFEST_LOGGER.info("file_title_map: %s", file_title_map)
    return file_title_map


def get_new_zipfile_name(doi, filename_pattern):
    article_id = article_id_from_doi(doi)
    new_zipfile_name = None
    if filename_pattern:
        new_zipfile_name = filename_pattern.format(article_id=article_id)
    return new_zipfile_name


def gen_new_zipfile(doi, poa_config):
    filename_pattern = poa_config.get("zipfile_pattern")
    new_zipfile_name = get_new_zipfile_name(doi, filename_pattern)
    new_zipfile_name_plus_path = poa_config.get("tmp_dir") + "/" + new_zipfile_name
    return new_zipfile_name_plus_path


def move_files_into_new_zipfile(
    current_zipfile, file_title_map, new_zipfile, doi, poa_config
):
    filename_pattern = poa_config.get("filename_pattern")
    for name in file_title_map:
        title = file_title_map[name]
        new_name = gen_new_name_for_file(name, title, doi, filename_pattern)

        file_from_zip = current_zipfile.read(name)
        temp_file_name = poa_config.get("tmp_dir") + "/" + "temp_transfer"
        with open(temp_file_name, "wb") as file_p:
            file_p.write(file_from_zip)
        add_file_to_zipfile(new_zipfile, temp_file_name, new_name)


def add_file_to_zipfile(new_zipfile, name, new_name):
    """
    Simple add a file to a zip file
    """
    if not new_zipfile or not name or not new_name:
        return
    new_zipfile.write(name, new_name)


def extract_pdf_from_zip(name, current_zipfile, decap_name, poa_config):
    # we extract the pdf from the zipfile
    file_from_zip = current_zipfile.read(name)
    decap_name_plus_path = poa_config.get("tmp_dir") + "/" + decap_name
    # we save the pdf to a local file
    with open(decap_name_plus_path, "wb") as temp_file:
        temp_file.write(file_from_zip)
    return decap_name_plus_path


def pdf_new_name(pdf_name, title, doi, poa_config):
    new_name = gen_new_name_for_file(
        pdf_name, title, doi, poa_config.get("filename_pattern")
    )
    LOGGER.info("new_name: %s", new_name)
    return new_name


def pdf_decap_name(new_name):
    return "decap_" + new_name


def pdf_details(file_title_map):
    "find the PDF file name in the zip file map"
    pdf_name = None
    title = None
    for name in file_title_map:
        file_title = file_title_map[name]
        if file_title == "Merged PDF":
            LOGGER.info("title: %s", title)
            pdf_name = name
            title = file_title
    return pdf_name, title


def decap_the_pdf(decap_name_plus_path, poa_config):
    decap_status = None
    try:
        # pass the local file path, and the path to a temp dir, to the decapitation script
        decap_status = func_timeout(
            PDF_DECAPITATE_TIMEOUT,
            decapitate_pdf_with_error_check,
            args=(
                decap_name_plus_path,
                poa_config.get("decapitate_pdf_dir") + os.sep,
                poa_config,
            ),
        )
    except FunctionTimedOut:
        decap_status = False
        LOGGER.error(
            "PDF decap did not finish within %s seconds", PDF_DECAPITATE_TIMEOUT
        )
    return decap_status


def copy_pdf_to_output_dir(
    file_title_map, output_dir, doi, current_zipfile, poa_config
):
    """
    we will attempt to generate a headless pdf and move this pdf
    to the output directory.

    if this headless creation fails, we will raise an error in the log file

    the function that we call to decapitate the pdf is contained in decapitate_pdf.py.
    It manages some error handline, and tries to determine whether the pdf
    cover content has been cleanly removed.
    """

    name, title = pdf_details(file_title_map)
    new_name = pdf_new_name(name, title, doi, poa_config)
    decap_name = pdf_decap_name(new_name)
    decap_name_plus_path = extract_pdf_from_zip(
        name, current_zipfile, decap_name, poa_config
    )

    decap_status = decap_the_pdf(decap_name_plus_path, poa_config)

    if decap_status:
        # pass the local file path, and the path to a temp dir, to the decapiation script
        try:
            file_content = None
            pdf_file_name = os.path.join(
                poa_config.get("decapitate_pdf_dir"), decap_name
            )
            with open(pdf_file_name, "rb") as open_file:
                file_content = open_file.read()
            if file_content:
                with open(os.path.join(output_dir, new_name), "wb") as out_handler:
                    out_handler.write(file_content)
        except IOError as ioe:
            # The decap may return true but the file does not exist for some reason
            #  allow the transformation to continue in order to processes the supplementary files
            LOGGER.error("decap returned true but the pdf file is missing %s: %s", new_name, ioe)
    else:
        # if the decapitation script has failed
        LOGGER.error("could not decapitate %s", new_name)


def remove_pdf_from_file_title_map(file_title_map):
    new_map = {}
    for name in file_title_map:
        title = file_title_map[name]
        if title == "Merged PDF":
            continue
        new_map[name] = title
    return new_map


def move_new_zipfile(doi, poa_config):
    filename_pattern = poa_config.get("zipfile_pattern")
    new_zipfile_name = get_new_zipfile_name(doi, filename_pattern)
    new_zipfile_name_plus_path = poa_config.get("tmp_dir") + "/" + new_zipfile_name
    shutil.move(
        new_zipfile_name_plus_path,
        poa_config.get("output_dir") + "/" + new_zipfile_name,
    )


def process_zipfile(zipfile_name, poa_config=None):
    # configuration can be passed in or parsed from the config_section
    if not poa_config:
        poa_config = parse_raw_config(raw_config(None))

    # open the zip file
    with zipfile.ZipFile(zipfile_name, "r") as current_zipfile:
        doi = get_doi_from_zipfile(current_zipfile)
        file_title_map = get_filename_new_title_map(current_zipfile)
        copy_pdf_to_output_dir(
            file_title_map, poa_config.get("output_dir"), doi, current_zipfile, poa_config
        )
        pdfless_file_title_map = remove_pdf_from_file_title_map(file_title_map)

        # supplements zip file
        new_zipfile_path = gen_new_zipfile(doi, poa_config)
        with zipfile.ZipFile(new_zipfile_path, "w") as new_zipfile:
            move_files_into_new_zipfile(
                current_zipfile, pdfless_file_title_map, new_zipfile, doi, poa_config
            )

    move_new_zipfile(doi, poa_config)
    return True



import unittest
import configparser
from packagepoa import conf


def build_config_for_testing(value_name, value):
    "build a config object with one value and return the defaults for testing"
    config = configparser.ConfigParser(interpolation=None)
    config["DEFAULT"][value_name] = value
    new_config = config["DEFAULT"]
    return new_config


class TestConf(unittest.TestCase):
    def setUp(self):
        # save conf values prior to testing
        self.old_boolean_values = conf.BOOLEAN_VALUES
        self.old_int_values = conf.INT_VALUES
        self.old_list_values = conf.LIST_VALUES

    def tearDown(self):
        # reset the conf values
        conf.BOOLEAN_VALUES = self.old_boolean_values
        conf.INT_VALUES = self.old_int_values
        conf.LIST_VALUES = self.old_list_values

    def test_load_config(self):
        "test loading when no config file is specified for test coverage"
        self.assertIsNotNone(conf.load_config())

    def test_boolean_config(self):
        "test parsing a boolean value"
        value = "True"
        value_name = "test"
        expected = True
        config = build_config_for_testing(value_name, value)
        self.assertEqual(conf.boolean_config(config, value_name), expected)

    def test_int_config(self):
        "test parsing an int value"
        value = "42"
        value_name = "test"
        expected = 42
        config = build_config_for_testing(value_name, value)
        self.assertEqual(conf.int_config(config, value_name), expected)

    def test_list_config(self):
        "test parsing a list value"
        value = "[1,1,2,3,5]"
        value_name = "test"
        expected = [1, 1, 2, 3, 5]
        config = build_config_for_testing(value_name, value)
        self.assertEqual(conf.list_config(config, value_name), expected)

    def test_parse_raw_config(self):
        "test parsing a raw config of all the different value types"
        # override the library values
        conf.BOOLEAN_VALUES = ["test_boolean"]
        conf.INT_VALUES = ["test_int"]
        conf.LIST_VALUES = ["test_list"]
        # build a config object
        config = configparser.ConfigParser(interpolation=None)
        config["DEFAULT"]["test_boolean"] = "True"
        config["DEFAULT"]["test_int"] = "42"
        config["DEFAULT"]["test_list"] = "[1,1,2,3,5]"
        # parse the raw config for coverage
        config_dict = conf.parse_raw_config(config["DEFAULT"])
        # assert some things
        self.assertIsNotNone(config_dict)
        self.assertEqual(config_dict.get("test_boolean"), True)
        self.assertEqual(config_dict.get("test_int"), 42)
        self.assertEqual(config_dict.get("test_list"), [1, 1, 2, 3, 5])
